let port;
let started = false;
let tabs = new Set();
const pendingRequests = new Map(); // Global pendingRequests Map

chrome.action.setIcon({ path: "/icons/ufr_icon_off_128.png" });

chrome.tabs.onUpdated.addListener((tabId, changeInfo, tab) => {
  if (changeInfo.status === 'complete') {
    chrome.tabs.query({ active: true, currentWindow: true }, (queriedTabs) => {
      const currentTab = queriedTabs[0];
      if (!currentTab) return;

      chrome.tabs.sendMessage(currentTab.id, { method: "getHTML" },
        (results) => {
          if (chrome.runtime.lastError) {
            console.warn("getHtml error: ", chrome.runtime.lastError.message);
            return;
          }

          if (typeof results !== "undefined") {
            const response = results.data;
            if (response === 'yes') {
              tabs.add(currentTab.id);
              if (!started) {
                port = chrome.runtime.connectNative('ufr.dlogic.chrome');
                started = true;

                if (port !== "undefined") {
                  port.onDisconnect.addListener(() => {
                    console.info('Disconnected.');
                    started = false;
                    pendingRequests.clear(); // Clear pending requests on disconnect
                  });
                }

                port.onMessage.addListener((msg1) => {
                  try {

                    msg1 = msg1.trim();
                    msg1 = atob(msg1);
                    msg1 = msg1.replace(/%%/g, '"');
                    msg1 = msg1.replace(/&&x/g, '\\x');

                    let result = {};
                    if (msg1 === 'Unknown command') {
                      result['Status'] = 'Unknown command';
                    } else {
                      const spl = msg1.split('||');
                      for (let i = 0; i < spl.length; i++) {
                        const keyVal = spl[i].split(' -> ');
                        if (keyVal.length === 2) {
                          result[keyVal[0]] = keyVal[1];
                        }
                      }
                    }

                    // Get the first matching requestId from pendingRequests
                    const [command, requestId] = Array.from(pendingRequests.entries())[0] || [null, null];
                    if (requestId) {
                      result.requestId = requestId;
                      pendingRequests.delete(command);
                    }

                    chrome.tabs.query({ active: true, currentWindow: true }, (queriedRspTabs) => {
                      if (queriedRspTabs[0]) {
                        chrome.tabs.sendMessage(
                          queriedRspTabs[0].id,
                          result,
                          function (rsp) {
                            if (chrome.runtime.lastError) {
                              console.warn("Send response error:", chrome.runtime.lastError);
                            }
                          }
                        );
                      }
                    });
                  } catch (error) {
                    console.error("Error processing native host message:", error);
                  }
                });
              }
              chrome.action.setIcon({ path: "/icons/ufr_icon_on_128.png" });
            } else {
              chrome.action.setIcon({ path: "/icons/ufr_icon_off_128.png" });
            }
          }
        });
    });
  }
});

chrome.tabs.onActivated.addListener((activeInfo) => {
  if (tabs.has(activeInfo.tabId)) {
    chrome.action.setIcon({ path: "/icons/ufr_icon_on_128.png" });
  } else {
    chrome.action.setIcon({ path: "/icons/ufr_icon_off_128.png" });
  }
});

chrome.runtime.onMessage.addListener((msg, sender, sendResponse) => {
  try {

    if (!port || !started) {
      console.error("Native host not connected");
      return;
    }

    // Handle both old and new message formats
    let processedCommand;
    let requestId;

    if (typeof msg === 'object' && msg.command) {
      processedCommand = msg.command;
      requestId = msg.requestId;
    } else {
      processedCommand = msg;
      requestId = Date.now().toString(36);
    }

    processedCommand = processedCommand.replace(/"/g, '%%').replace(/\\x/g, '&&x');

    if (processedCommand.length % 8 === 0) {
      processedCommand += ' ';
    }

    // Store the requestId for correlation
    pendingRequests.set(processedCommand, requestId);

    // Send command to native host
    port.postMessage(processedCommand);
  } catch (error) {
    console.error("Error processing message:", error);
  }
});

chrome.tabs.onRemoved.addListener((tabId, removed) => {
  tabs.delete(tabId);

  if (tabs.size === 0) {
    if (port && started) {
      port.postMessage('#EXIT');
      chrome.action.setIcon({ path: "/icons/ufr_icon_off_128.png" });
      started = false;
      pendingRequests.clear();
    }
  }
});

chrome.runtime.onConnect.addListener((externalPort) => {
  externalPort.onDisconnect.addListener(() => {
    console.info('Disconnected.');
    started = false;
    pendingRequests.clear();
  });
});