const ufrAttribute = document.getElementsByTagName("html")[0].getAttribute("ufr");
if (ufrAttribute != null) {
    // Inject the script with improved methods for communication
    let actualCode = `
    function uFR_Request(command) {
        const requestId = Date.now().toString(36) + Math.random().toString(36).substr(2);
        
        return new Promise((resolve, reject) => {
            let timeoutId;
            
            const handleResponse = (event) => {
                try {
                    const response = JSON.parse(window.sessionStorage.response);
                    if (response.requestId === requestId || !response.requestId) {
                        clearTimeout(timeoutId);
                        document.removeEventListener("get-ufr", handleResponse);
                        
                        if (response.Status === 'Unknown command') {
                            reject(new Error('Unknown command'));
                        } else {
                            delete response.requestId;
                            resolve(response);
                        }
                    }
                } catch (error) {
                    console.error("Error processing response:", error);
                    reject(error);
                }
            };
            
            timeoutId = setTimeout(() => {
                document.removeEventListener("get-ufr", handleResponse);
                reject(new Error("Request timeout"));
            }, 5000);
            
            try {
                document.addEventListener("get-ufr", handleResponse);
                
                const event = new CustomEvent("send-ufr", {
                    detail: {
                        command: command,
                        requestId: requestId
                    },
                    bubbles: false,
                    cancelable: true
                });
                
                document.dispatchEvent(event);
            } catch (error) {
                clearTimeout(timeoutId);
                document.removeEventListener("get-ufr", handleResponse);
                reject(error);
            }
        });
    }

    // For compatibility with older code
    function ufRequest(command, callback) {
        uFR_Request(command)
            .then(response => callback && callback(response))
            .catch(error => console.error("ufRequest error:", error));
    }
    // For compatibility with older code
    function ufResponse() {
        let response = JSON.parse(window.sessionStorage["response"]);
        delete response.requestId;
        return response;
    }

    // For compatibility with older code
    function request(command, callback) {
        uFR_Request(command)
            .then(response => callback && callback(response))
            .catch(error => console.error("request error:", error));
    }
    `;

    const script = document.createElement('script');
    script.textContent = actualCode;
    (document.head || document.documentElement).appendChild(script);

    // Listen for send-ufr events from the web page
    document.addEventListener("send-ufr", function (data) {
        let request = data.detail;
        browser.runtime.sendMessage(request);
    });

    // Handle messages from the background script
    browser.runtime.onMessage.addListener(function (response, sender, sendResponse) {
        try {
            window.sessionStorage["response"] = JSON.stringify(response);

            // Dispatch event to notify web page that a response is available
            const event = new CustomEvent("get-ufr", {
                bubbles: false,
                cancelable: true
            });

            document.dispatchEvent(event);
        } catch (error) {
            console.error("Error processing message from background:", error);
        }
    });
}

// Listen for getHTML requests from the background script
browser.runtime.onMessage.addListener(function (request, sender, sendResponse) {
    if (request.method === "getHTML") {
        if (document.getElementsByTagName("html")[0].getAttribute("ufr") != null) {
            sendResponse({ data: "yes" });
        } else {
            sendResponse({ data: "no" });
        }
    } else {
        sendResponse({ data: "ok" });
    }

    return true;
});