function uFR_Request(command) {
    const requestId = Date.now().toString(36) + Math.random().toString(36).substr(2);
    
    return new Promise((resolve, reject) => {
        let timeoutId;
        
        const handleResponse = (event) => {
            try {
                const response = JSON.parse(window.sessionStorage.response);
                console.log("Received response:", response); // Debug log
                
                if (response.requestId === requestId || !response.requestId) {
                    clearTimeout(timeoutId);
                    document.removeEventListener("get-ufr", handleResponse);
                    
                    if (response.Status === 'Unknown command') {
                        reject(new Error('Unknown command'));
                    } else {
                        delete response.requestId;
                        console.log("Returning response...", response); // Debug log
                        resolve(response);
                    }
                }
            } catch (error) {
                console.error("Error processing response:", error);
                reject(error);
            }
        };
        
        timeoutId = setTimeout(() => {
            document.removeEventListener("get-ufr", handleResponse);
            reject(new Error("Request timeout"));
        }, 5000);
        
        try {
            document.addEventListener("get-ufr", handleResponse);
            
            const event = new CustomEvent("send-ufr", {
                detail: {
                    command: command,
                    requestId: requestId
                },
                bubbles: false,
                cancelable: true
            });
            
            console.log("Sending request:", { command, requestId }); // Debug log
            document.dispatchEvent(event);
        } catch (error) {
            clearTimeout(timeoutId);
            document.removeEventListener("get-ufr", handleResponse);
            reject(error);
        }
    });
}

// For compatibility with older code
function ufRequest(command, callback) {
    uFR_Request(command)
        .then(response => callback && callback(response))
        .catch(error => console.error("ufRequest error:", error));
}
// For compatibility with older code
function ufResponse() {
    let response = JSON.parse(window.sessionStorage["response"]);
    delete response.requestId;
    return response;
}

// For compatibility with older code
function request(command, callback) {
    uFR_Request(command)
        .then(response => callback && callback(response))
        .catch(error => console.error("request error:", error));
}